"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("@aaa-backend-stack/logger");
var logger = logger_1.getChildLogger("@aaa-backend-stack/graphql-rest-bindings");
var _ = require("lodash");
var graphql_1 = require("@aaa-backend-stack/graphql");
// used in only production.
exports.UNHANDLED_SERVER_ERROR = graphql_1.createError("UNHANDLED_SERVER_ERROR", {
    message: "The operations team has been notified."
});
function generateGraphqlRequestErrorHandler(request, options) {
    return function formatError(error) {
        var originalError = error.originalError;
        // Log unknown errors as fatal errors!
        if (originalError instanceof graphql_1.GraphQLError === true // directly catch GraphQLErrors
            || graphql_1.isApolloErrorInstance(originalError) === true) { // apolloErrors
            // ok, known error
            logger.error({
                error: error,
                message: error.message,
                locations: error.locations,
                stack: error.stack,
                requestPayload: request ? request.payload : null
            }, "GraphQL defined serverside error");
        }
        else if (_.isUndefined(originalError) === true) {
            // directly catch clientside errors (those have no originalError set!)
            logger.error({
                error: error,
                message: error.message,
                locations: error.locations,
                stack: error.stack,
                requestPayload: request ? request.payload : null
            }, "GraphQL clientside error");
        }
        else {
            // fatal error!
            logger.fatal({
                error: error,
                message: error.message,
                locations: error.locations,
                stack: error.stack,
                requestPayload: request ? request.payload : null
            }, "GraphQL unhandled serverside fatal error");
            if (options.includeServerStackTracesInClientErrors === false) {
                // never show the stack to clients as it might contain sensitive information.
                // only enabled if the service is running in production mode.
                return graphql_1.formatApolloError(new exports.UNHANDLED_SERVER_ERROR());
            }
        }
        // else format error through apollo-error and pass to the client... 
        return graphql_1.formatApolloError(error);
    };
}
exports.generateGraphqlRequestErrorHandler = generateGraphqlRequestErrorHandler;
//# sourceMappingURL=generateGraphqlRequestErrorHandler.js.map